@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -50,15 +50,6 @@ memlistConstruct_IMPL
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
-#define CLEAR_HAL_ATTR(a) \
-    a = (a & ~(DRF_SHIFTMASK(NVOS32_ATTR_COMPR) | \
-               DRF_SHIFTMASK(NVOS32_ATTR_TILED) | \
-               DRF_SHIFTMASK(NVOS32_ATTR_ZCULL)));
-
-#define CLEAR_HAL_ATTR2(a) \
-    a = (a & ~(DRF_SHIFTMASK(NVOS32_ATTR2_ZBC) | \
-               DRF_SHIFTMASK(NVOS32_ATTR2_GPU_CACHEABLE)));
-
     NV_MEMORY_LIST_ALLOCATION_PARAMS *pAllocParams;
     RsResourceRef                    *pResourceRef = pCallContext->pResourceRef;
     NvHandle                          hClient = pCallContext->pClient->hClient;
@@ -110,22 +101,6 @@ memlistConstruct_IMPL
     }
 continue_alloc_object:
 
-    //
-    // These classes are used by the vGPU support to create memory objects for memory
-    // assigned to a guest VM.  The NV01_MEMORY_LIST_OBJECT case creates an object
-    // whose pages are based on another object's pages.
-    //
-    if ((externalClassId == NV01_MEMORY_LIST_FBMEM) || (externalClassId == NV01_MEMORY_LIST_OBJECT))
-    {
-        if (!(rmclientIsAdminByHandle(hClient, privLevel) || hypervisorCheckForObjectAccess(hClient)))
-            return NV_ERR_INSUFFICIENT_PERMISSIONS;
-    }
-    else
-    {
-        if (!rmclientIsAdminByHandle(hClient, privLevel))
-            return NV_ERR_INSUFFICIENT_PERMISSIONS;
-    }
-
     if (DRF_VAL(OS02, _FLAGS, _COHERENCY, pAllocParams->flagsOs02) == NVOS02_FLAGS_COHERENCY_UNCACHED)
         Cache = NV_MEMORY_UNCACHED;
     else if (DRF_VAL(OS02, _FLAGS, _COHERENCY, pAllocParams->flagsOs02) == NVOS02_FLAGS_COHERENCY_CACHED)
@@ -288,11 +263,7 @@ continue_alloc_object:
     {
         NvU32 attr2 = 0;
 
-        if ((src_pMemDesc != NULL) || bContig)
-        {
-            NV_ASSERT(0);
-            return NV_ERR_NOT_SUPPORTED;
-        }
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, src_pMemDesc == NULL, NV_ERR_NOT_SUPPORTED);
 
         status = memdescCreate(&pMemDesc, pGpu, memSize, 0,
                                bContig, addressSpace, Cache,
@@ -310,7 +281,8 @@ continue_alloc_object:
 
         pPteArray = memdescGetPteArray(pMemDesc, AT_GPU);
 
-        if (!portSafeMulU32(sizeof(NvU64), pAllocParams->pageCount, &result))
+        if ((pAllocParams->pageCount > pMemDesc->PageCount) ||
+            !portSafeMulU32(sizeof(NvU64), pAllocParams->pageCount, &result))
         {
             memdescDestroy(pMemDesc);
             return NV_ERR_INVALID_ARGUMENT;
@@ -369,7 +341,8 @@ continue_alloc_object:
 
         // this will fake a memory allocation at
         // the OS driver interface level - and also set pMemDesc->Allocated
-        status = memdescAlloc(pMemDesc);
+        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_46,
+                        pMemDesc);
 
         if (status != NV_OK)
         {
@@ -411,6 +384,20 @@ continue_alloc_object:
         {
             memdescDestroy(pMemDesc);
         }
+        //
+        // We currently map all allocations that get registered with GSP
+        // since there is no mechanism to specify whether the registered
+        // allocation is ever accessed by GSP.
+        //
+        else if (RMCFG_FEATURE_PLATFORM_GSP)
+        {
+            // GSP only supports contiguous mappings
+            if (pAllocParams->pageCount == 1)
+            {
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                    memCreateKernelMapping(pMemory, NV_PROTECT_READ_WRITE, NV_FALSE));
+            }
+        }
     }
     else if (addressSpace == ADDR_FBMEM)
     {
@@ -418,15 +405,24 @@ continue_alloc_object:
         NvU64                              baseOffset          = 0;
         NvU64                              trueLength;
         NvU32                              hwResId             = 0;
-        NvU32                              pageSize            = 0;
+        NvU64                              pageSize            = 0;
         RM_ATTR_PAGE_SIZE                  pageSizeAttr;
         FB_ALLOC_INFO                     *pFbAllocInfo        = NULL;
         FB_ALLOC_PAGE_FORMAT              *pFbAllocPageFormat  = NULL;
         HWRESOURCE_INFO                   *pHwResource         = NULL;
         MemoryManager                     *pMemoryManager      = GPU_GET_MEMORY_MANAGER(pGpu);
         KernelGmmu                        *pKernelGmmu         = GPU_GET_KERNEL_GMMU(pGpu);
-        Heap                              *pHeap               = vidmemGetHeap(pGpu, hClient, NV_FALSE);
+        Heap                              *pHeap;
         NvBool                             bCallingContextPlugin;
+        RsResourceRef                     *pDeviceRef;
+
+        NV_ASSERT_OK_OR_RETURN(
+            refFindAncestorOfType(pResourceRef, classId(Device), &pDeviceRef));
+
+        pHeap = vidmemGetHeap(pGpu,
+                              dynamicCast(pDeviceRef->pResource, Device),
+                              NV_FALSE,
+                              NV_FALSE);
 
         //
         // When guest RM client doesn't subscribe to MIG partition and requests for vidmem allocation
@@ -579,6 +575,12 @@ continue_alloc_object:
             // only a kernel client can request for a protected allocation
             if (pFbAllocInfo->pageFormat->flags & NVOS32_ALLOC_FLAGS_ALLOCATE_KERNEL_PRIVILEGED)
             {
+                if (privLevel < RS_PRIV_LEVEL_KERNEL)
+                {
+                    status = NV_ERR_INSUFFICIENT_PERMISSIONS;
+                    NV_PRINTF(LEVEL_ERROR, "only kernel clients may request for a protected allocation\n");
+                    goto done_fbmem;
+                }
                 pFbAllocInfo->bIsKernelAlloc = NV_TRUE;
             }
 
@@ -668,15 +670,6 @@ continue_alloc_object:
                       pFbAllocInfo->hwResId);
 
             hwResId = pFbAllocInfo->hwResId;
-
-            //
-            // For Linux Guest, we allocate hardware resources on the host through
-            // NV01_MEMORY_LIST_OBJECT or NV01_MEMORY_LIST_FBMEM class objects.
-            // isGuestAllocated flag when set TRUE indicates that hardware resource
-            // is allocated for Linux Guest and we will free it in
-            // memDestructCommon.
-            //
-            pHwResource->isGuestAllocated = NV_TRUE;
         }
         else if (src_hHwResHandle != 0)
         {
@@ -717,6 +710,7 @@ continue_alloc_object:
             case RM_ATTR_PAGE_SIZE_DEFAULT:
                 NV_PRINTF(LEVEL_INFO, "page size default doesn't have any impact \n");
                 break;
+            case RM_ATTR_PAGE_SIZE_256GB:
             case RM_ATTR_PAGE_SIZE_INVALID:
                 NV_PRINTF(LEVEL_INFO, "unexpected pageSizeAttr = 0x%x\n", pageSizeAttr);
                 status = NV_ERR_INVALID_STATE;
@@ -765,6 +759,8 @@ done_fbmem:
                 pFbAllocInfo->pageFormat->type = pAllocParams->type;
                 pFbAllocInfo->hwResId          = hwResId;
                 pFbAllocInfo->size             = memSize;
+                pFbAllocInfo->hClient          = hClient;
+                pFbAllocInfo->hDevice          = hParent;
 
                 memmgrFreeHwResources(pGpu, pMemoryManager, pFbAllocInfo);
             }
